// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/driver/device.c $
// $Rev: 54540 $
// $Date: 2024-05-28 09:45:15 -0500 (Tue, 28 May 2024) $

// DIO32: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model	Vendor	Device	SubVen	SubDev	type

	{ "DIO32A",	0x10B5, 0x9056, 0x10B5, 0x2706,	GSC_DEV_TYPE_DIO32	},
	{ NULL }
};



//*****************************************************************************
static void _bus_width_compute(dev_data_t* dev)
{
	u16	sid;

	sid	= os_reg_pci_rx_u16(dev, 0, 2);

	if (sid == 0x9056)
		dev->cache.bus_width	= 64;
	else
		dev->cache.bus_width	= 32;
}



//*****************************************************************************
static void _form_factor_compute(dev_data_t* dev)
{
	dev->cache.form_factor	= GSC_FIELD_DECODE(dev->cache.gsc_frr_32, 27, 24);
}



//*****************************************************************************
static void _ports_compute(dev_data_t* dev)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
	bsr	= (bsr & 0xEFFF0000) >> 16;

	dev->cache.port_qty	= bsr;
}



//*****************************************************************************
static void _proc_id_str_compute(dev_data_t* dev)
{
	if (dev->cache.user_jumper_qty)
		sprintf(dev->proc_id_str, "0x%lX", (long) dev->cache.user_jumper_value);
	else
		strcpy(dev->proc_id_str, "none");
}



//*****************************************************************************
static void _user_jumpers_compute(dev_data_t* dev)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
	bsr	&= 0xF;

	dev->cache.user_jumper_qty		= 4;
	dev->cache.user_jumper_sense	= 0;
	dev->cache.user_jumper_value	= bsr;
}



//*****************************************************************************
static void _xcvr_compute(dev_data_t* dev)
{
	dev->cache.xcvr_type	= DIO32_QUERY_XCVR_RS485;
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	int	i;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(DIO32_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_BCR));
		dev->vaddr.gsc_bsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_BSR));

		dev->vaddr.gsc_dgcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_DGCR));
		dev->vaddr.gsc_dgir_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_DGIR));

		dev->vaddr.gsc_fr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_FR));
		dev->vaddr.gsc_frr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_FRR));

		dev->vaddr.gsc_hlier_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_HLIER));
		dev->vaddr.gsc_hliscr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_HLISCR));

		dev->vaddr.gsc_iodr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_IODR));
		dev->vaddr.gsc_itr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_ITR));

		dev->vaddr.gsc_lhier_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_LHIER));
		dev->vaddr.gsc_lhiscr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_LHISCR));

		dev->vaddr.gsc_ocr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_OCR));
		dev->vaddr.gsc_otsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_OTSR));
		dev->vaddr.gsc_ovr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_OVR));

		dev->vaddr.gsc_pcifr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC1FR));
		dev->vaddr.gsc_pc1pwmdr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC1PWMDR));
		dev->vaddr.gsc_pc1r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC1R));
		dev->vaddr.gsc_pc2fr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC2FR));
		dev->vaddr.gsc_pc2pwmdr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC2PWMDR));
		dev->vaddr.gsc_pc2r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC2R));
		dev->vaddr.gsc_pc3fr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC3FR));
		dev->vaddr.gsc_pc3pwmdr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC3PWMDR));
		dev->vaddr.gsc_pc3r_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PC3R));
		dev->vaddr.gsc_pir_32		= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_PIR));

		dev->vaddr.gsc_hl_out_msk[0]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_D00HLOMR));
		dev->vaddr.gsc_hl_out_tst[0]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_D00HLOTSR));
		dev->vaddr.gsc_hl_out_val[0]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO32_GSC_D00HLOVR));

		for (i = 0; i <= 31; i++)
		{
			dev->vaddr.gsc_hl_out_msk[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00HLOMR));
			dev->vaddr.gsc_hl_out_tst[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00HLOTSR));
			dev->vaddr.gsc_hl_out_val[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00HLOVR));

			dev->vaddr.gsc_lh_out_msk[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00LHOMR));
			dev->vaddr.gsc_lh_out_tst[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00LHOTSR));
			dev->vaddr.gsc_lh_out_val[i]	= GSC_VADDR(dev, 0x20 * i + GSC_REG_OFFSET(DIO32_GSC_D00LHOVR));
		}

		// Data cache initialization.

		dev->cache.gsc_fr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_fr_32);
		dev->cache.gsc_frr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_frr_32);

		_form_factor_compute(dev);
		_bus_width_compute(dev);
		_user_jumpers_compute(dev);
		_xcvr_compute(dev);
		_proc_id_str_compute(dev);	// must be after jumpers
		_ports_compute(dev);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);

		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


